import boto3
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_S3, AWSCredentials_SQS


def get_aws_s3_client(credentials: AWSCredentials_S3):
    """Get the AWS S3 client

    Parameters
    ----------
    credentials: Credentials for accessing the S3 service
    Returns
    -------
    """
    return boto3.client('s3',
                        aws_access_key_id=credentials.aws_access_key,
                        aws_secret_access_key=credentials.aws_secret_access_key,
                        region_name=credentials.aws_region)


def get_aws_sqs_client(credentials: AWSCredentials_SQS):
    """Get the AWS SQS client

    Parameters
    ----------
    credentials: Credentials for accessing the SQS service

    Returns
    -------

    """
    return boto3.client('sqs',
                        aws_access_key_id=credentials.aws_access_key,
                        aws_secret_access_key=credentials.aws_secret_access_key,
                        region_name=credentials.aws_region)












from typing import Any
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_SQS
from navalmartin_mir_aws_utils.boto3_client import get_aws_sqs_client
from navalmartin_mir_aws_utils.sqs_queue_config import SQSMessageConfig


def send_sqs_message(sqs_credentials: AWSCredentials_SQS,
                     sqs_msg: SQSMessageConfig, sqs_client: Any = None) -> dict:
    """Send a message to the SQS queue specified
    under the sqs credentials

    Parameters
    ----------
    sqs_credentials: The credentials to use to send the message
    sqs_msg: The message configuration

    Returns
    -------
    A dictionary with the response attributes
    """

    if sqs_client is not None:
        response = sqs_client.send_message(QueueUrl=sqs_credentials.queue_url,
                                           MessageBody=sqs_msg.message_body,
                                           # DelaySeconds=sqs_msg.delay_seconds,
                                           MessageAttributes=sqs_msg.message_attributes if sqs_msg.message_attributes is not None else {},
                                           MessageGroupId=sqs_msg.message_group_id,
                                           MessageDeduplicationId=sqs_msg.message_deduplication_id)
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response: dict = new_sqs_client.send_message(QueueUrl=sqs_credentials.queue_url,
                                                 MessageBody=sqs_msg.message_body,
                                                 # DelaySeconds=sqs_msg.delay_seconds,
                                                 MessageAttributes=sqs_msg.message_attributes if sqs_msg.message_attributes is not None else {},
                                                 MessageGroupId=sqs_msg.message_group_id,
                                                 MessageDeduplicationId=sqs_msg.message_deduplication_id)
    return response


def read_one_sqs_message(sqs_credentials: AWSCredentials_SQS,
                         sqs_client: Any = None) -> dict:
    """

    Parameters
    ----------
    sqs_client: The SQS client to use for the operation
    sqs_credentials: The credentials to use for the queue

    Returns
    -------

    A dictionary of the response
    """

    if sqs_client is not None:
        response = sqs_client.receive_message(QueueUrl=sqs_credentials.queue_url,
                                              MaxNumberOfMessages=1)
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response = new_sqs_client.receive_message(QueueUrl=sqs_credentials.queue_url,
                                              MaxNumberOfMessages=1)

    return response


def delete_sqs_message(sqs_credentials: AWSCredentials_SQS, receipt_handle: str,
                       sqs_client: Any = None) -> dict:
    """Deletes the message from the queue

    Parameters
    ----------
    sqs_client: The SQS client to use for the operation
    sqs_credentials: The credentials to use for the queue
    receipt_handle: This is the handle received when receiving the message

    Returns
    -------

    """

    if sqs_client is not None:
        response = sqs_client.delete_message(QueueUrl=sqs_credentials.queue_url,
                                             ReceiptHandle=receipt_handle)
        return response

    new_sqs_client = get_aws_sqs_client(credentials=sqs_credentials)
    response = new_sqs_client.delete_message(QueueUrl=sqs_credentials.queue_url,
                                             ReceiptHandle=receipt_handle)
    return response



from navalmartin_mir_aws_utils.image_s3_batch import ImagePathBatch
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_S3

AWS_REGION = "YOUR_AWS_REGION"
AWS_S3_BUCKET_NAME = "YOUR_AWS_S3_BUCKET_NAME"
AWS_ACCESS_KEY = "YOUR_AWS_ACCESS_KEY"
AWS_SECRET_ACCESS_KEY = "YOUR_AWS_SECRET_ACCESS_KEY"
IMAGE_STR_TYPES = ('.jpg', '.png', '.jpeg')

AWS_REGION = ""
AWS_S3_BUCKET_NAME = ""
AWS_ACCESS_KEY = ""
AWS_SECRET_ACCESS_KEY = ""
MONGO_DB_URL=""

if __name__ == '__main__':
    aws_s3_credentials = AWSCredentials_S3(aws_s3_bucket_name=AWS_S3_BUCKET_NAME,
                                           aws_region=AWS_REGION,
                                           aws_access_key=AWS_ACCESS_KEY,
                                           aws_secret_access_key=AWS_SECRET_ACCESS_KEY)

    image_prefix = "63b5bc8cc5f1cdcad7ef54e7/2/" #'some/image/prefix/'
    image_batch = ImagePathBatch(s3_credentials=aws_s3_credentials)

    # read the images
    image_batch.read(prefixes=(image_prefix,),
                     valid_image_extensions=IMAGE_STR_TYPES,
                     delimiter='/')

    for image_file in image_batch:
        print(image_file)



"""module aws_credentials. Utility class to move around
credentials. This class is not used to load the
credentials. It is the application's duty to do so

"""


class AWSCredentials(object):
    def __init__(self, aws_client_name: str  = None,
                 aws_access_key: str = None,
                 aws_secret_access_key: str  = None):

        self.aws_client_name: str = aws_client_name
        self.aws_access_key: str = aws_access_key
        self.aws_secret_access_key = aws_secret_access_key


class AWSCredentials_S3(AWSCredentials):
    def __init__(self,  aws_s3_bucket_name: str, aws_region: str,
                 aws_access_key: str, aws_secret_access_key: str):
        super(AWSCredentials_S3, self).__init__(aws_client_name='s3',
                                                aws_secret_access_key=aws_secret_access_key,
                                                aws_access_key=aws_access_key)
        self.aws_s3_bucket_name = aws_s3_bucket_name
        self.aws_region = aws_region


class AWSCredentials_SQS(AWSCredentials):
    def __init__(self,  aws_sqs_queue_name: str, aws_region: str,
                 aws_access_key: str, aws_secret_access_key: str,
                 aws_queue_url: str):
        super(AWSCredentials_SQS, self).__init__(aws_client_name='sqs',
                                                 aws_secret_access_key=aws_secret_access_key,
                                                 aws_access_key=aws_access_key)
        self.aws_sqs_queue_name: str = aws_sqs_queue_name
        self.aws_region: str = aws_region
        self.queue_url: str = aws_queue_url





"""module image_s3_batch. Represents
a batch of images on S3

"""
import os
from typing import List, Any, Union
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_S3
from navalmartin_mir_aws_utils.boto3_client import get_aws_s3_client
from navalmartin_mir_aws_utils.s3_utils import expand_s3_iterator_contents, expand_s3_iterator_common_prefixes


class ImagePathBatch(object):

    def __init__(self, s3_credentials: AWSCredentials_S3, delimiter: str = '/',
                 do_build_client: bool = True):
        self.aws_bucket_credentials = s3_credentials
        self.images: List[dict] = []
        self.delimiter = delimiter
        self._current_pos: int = -1
        self._client: Any = None

        if do_build_client:
            self.build_client()

    def __len__(self):
        return len(self.images)

    def __iter__(self):
        self._current_pos = 0
        return self

    def __next__(self):
        if len(self.images) == 0:
            raise StopIteration

        if self._current_pos < len(self.images):
            result = self.images[self._current_pos]
            self._current_pos += 1
            return result
        else:
            self._current_pos = -1
            raise StopIteration

    def build_client(self) -> Any:
        self._client = get_aws_s3_client(credentials=self.aws_bucket_credentials)

    def load_from_list(self, images: List[str], delimiter="/"):
        self.images = images
        self.delimiter = delimiter

    def read(self, prefixes: tuple, valid_image_extensions: tuple,
             delimiter: str = '/'):

        self.delimiter = delimiter

        # get the object that lists the objects
        # on S3
        paginator = self._client.get_paginator('list_objects')

        for prefix in prefixes:
            s3_iterator = paginator.paginate(Bucket=self.aws_bucket_credentials.aws_s3_bucket_name,
                                             Delimiter=self.delimiter,
                                             Prefix=prefix)

            contents = expand_s3_iterator_contents(s3_iterator)

            if len(contents) != 0:
                self.read_from_contents(contents=contents,
                                        valid_image_extensions=valid_image_extensions)

            common_prefixes = expand_s3_iterator_common_prefixes(s3_iterator=s3_iterator)
            if len(common_prefixes) != 0:
                self.read_from_common_prefixes(common_prefixes=common_prefixes,
                                               valid_image_extensions=valid_image_extensions)

    def copy_to(self, s3_credentials_to: AWSCredentials_S3) -> None:
        """Copy the images in the bucket to the S3 bucket specified
        by the provided credentials

        Parameters
        ----------
        s3_credentials_to: The credentials to use for the bucket to copy

        Returns
        -------

        """

        if len(self.images) == 0:
            print("WARNING: Image batch is empty...")
            return

        raise NotImplementedError("The function is not implemented")

    def get_object(self, key: Union[int, str]) -> Any:
        """Returns the object specified with the given key.

        Parameters
        ----------
        key: The key of the object to return

        Returns
        -------

        """

        if type(key) == int:

            if key >= len(self.images):
                raise ValueError(f"Invalid key. Integer key {key} cannot be >= {len(self.images)}")

            key = self.images[key]
        elif type(key) == str:
            if key not in self.images:
                raise ValueError(f"key={key} not in {self.images}")
        else:
            raise ValueError(f"key type {type(key)} is not valie")

        if self._client is None:
            raise ValueError("S3 client is not built")

        file_byte_string = self._client.get_object(Bucket=self.aws_bucket_credentials.aws_s3_bucket_name,
                                                   Key=key)

        return file_byte_string

    def read_from_contents(self, contents: List[dict], valid_image_extensions: tuple) -> None:
        """Read the contents of the S3 bucket

        Parameters
        ----------
        contents: The contents to use for reading the images
        valid_image_extensions: The valid_image_extensions to look for

        Returns
        -------

        """

        n_extensions = len(valid_image_extensions)
        for content in contents:

            image = content.get('Key')

            if n_extensions != 0:

                img_extension = os.path.splitext(image)[1]
                if img_extension in valid_image_extensions:
                    self.images.append({'img': content.get('Key'),
                                        'bucket': self.aws_bucket_credentials.aws_s3_bucket_name})
            else:
                self.images.append({'img': content.get('Key'),
                                    'bucket': self.aws_bucket_credentials.aws_s3_bucket_name})

    def read_from_common_prefixes(self, common_prefixes: List[dict],
                                  valid_image_extensions: tuple) -> None:
        """Read from the common_prefixes

        Parameters
        ----------
        common_prefixes: The common_prefixes to use for reading
        valid_image_extensions: The valid_image_extensions to look for

        Returns
        -------

        """

        if self._client is None:
            raise ValueError("S3 client is not built")

        paginator = self._client.get_paginator('list_objects')
        for item in common_prefixes:
            prefix = item['Prefix']

            s3_iterator = paginator.paginate(Bucket=self.aws_bucket_credentials.aws_s3_bucket_name,
                                             Delimiter=self.delimiter,
                                             Prefix=prefix)

            contents = expand_s3_iterator_contents(s3_iterator)
            self.read_from_contents(contents=contents,
                                    valid_image_extensions=valid_image_extensions)



from typing import Any, List


def expand_s3_iterator_contents(s3_iterator: Any) -> List[dict]:
    """Get the contents of the provided S3 iterator

    Parameters
    ----------
    s3_iterator: The S3 iterator

    Returns
    -------
    A list of dictionaries
    """

    contents = s3_iterator.search('Contents')
    return [item for item in contents if item is not None]


def expand_s3_iterator_common_prefixes(s3_iterator: Any) -> List[dict]:
    """Get the common prefixes from the iterator

    Parameters
    ----------
    s3_iterator: The S3 iterator

    Returns
    -------
    A list of dictionaries
    """
    contents = s3_iterator.search('CommonPrefixes')
    return [item for item in contents if item is not None]
